@@ -18,8 +18,9 @@ try:
 except ImportError:
     import urllib2
 import json
+from . import util
 
-__version__ = '0.24'
+__version__ = '0.26'
 
 
 class __UnknownValue(object):
@@ -86,7 +87,7 @@ class System(object):
 
         #: List of all authors of this system, as a list of strings (last name
         #: followed by initials, e.g. "Smith AJ"). When writing out a file,
-        #: if this is list is empty, the set of all citation authors (see
+        #: if this list is empty, the set of all citation authors (see
         #: :attr:`Citation.authors`) is used instead.
         self.authors = []
 
@@ -211,6 +212,26 @@ class System(object):
         #: See :class:`~ihm.flr.FLRData`.
         self.flr_data = []
 
+    def _make_complete_assembly(self):
+        """Fill in the complete assembly with all asym units"""
+        # Clear out any existing components
+        self.complete_assembly[:] = []
+
+        # Include all asym units
+        for asym in self.asym_units:
+            self.complete_assembly.append(asym)
+
+    def _all_models(self):
+        """Iterate over all Models in the system"""
+        # todo: raise an error if a model is present in multiple groups
+        for group in self._all_model_groups():
+            seen_models = {}
+            for model in group:
+                if model in seen_models:
+                    continue
+                seen_models[model] = None
+                yield group, model
+
     def update_locations_in_repositories(self, repos):
         """Update all :class:`Location` objects in the system that lie within
            a checked-out :class:`Repository` to point to that repository.
@@ -283,17 +304,6 @@ class System(object):
                         yield edge.group_begin
                         yield edge.group_end
 
-    def _all_models(self):
-        """Iterate over all Models in the system"""
-        # todo: raise an error if a model is present in multiple groups
-        for group in self._all_model_groups():
-            seen_models = {}
-            for model in group:
-                if model in seen_models:
-                    continue
-                seen_models[model] = None
-                yield group, model
-
     def _all_representations(self):
         """Iterate over all Representations in the system.
            This includes all Representations referenced from other objects,
@@ -529,14 +539,26 @@ class System(object):
                 for comp in f._all_entities_or_asyms()),
             (d.asym_unit for d in self._all_densities())))
 
-    def _make_complete_assembly(self):
-        """Fill in the complete assembly with all asym units"""
-        # Clear out any existing components
-        self.complete_assembly[:] = []
+    def _before_write(self):
+        """Do any setup necessary before writing out to a file"""
+        # Here, we initialize all RestraintGroups by removing any assigned ID
+        for g in self.restraint_groups:
+            util._remove_id(g)
+        # Fill in complete assembly
+        self._make_complete_assembly()
 
-        # Include all asym units
-        for asym in self.asym_units:
-            self.complete_assembly.append(asym)
+    def _check_after_write(self):
+        """Make sure everything was successfully written"""
+        # Here, we check that all RestraintGroups were successfully dumped"""
+        for g in self.restraint_groups:
+            if len(g) > 0 and not hasattr(g, '_id'):
+                raise TypeError(
+                    "RestraintGroup(%s) contains an unsupported combination "
+                    "of Restraints. Due to limitations of the underlying "
+                    "dictionary, all objects in a RestraintGroup must be of "
+                    "the same type, and only certain types (currently only "
+                    "DerivedDistanceRestraint or PredictedContactRestraint) "
+                    "can be grouped." % g)
 
 
 class Software(object):
@@ -737,8 +759,9 @@ class ChemComp(object):
             emass = self._element_mass.get(m.group(1), None)
             if emass:
                 weight += emass * (int(m.group(2)) if m.group(2) else 1)
-            else:
+            elif m.group(1) != 'X':
                 # If element is unknown, weight is unknown too
+                # Element 'X' is used for GLX/ASX and has zero weight
                 return None
         return weight
 
@@ -864,7 +887,9 @@ class LPeptideAlphabet(Alphabet):
                   ('T', 'THR', 'THREONINE', 'C4 H9 N O3'),
                   ('V', 'VAL', 'VALINE', 'C5 H11 N O2'),
                   ('W', 'TRP', 'TRYPTOPHAN', 'C11 H12 N2 O2'),
-                  ('Y', 'TYR', 'TYROSINE', 'C9 H11 N O3')])
+                  ('Y', 'TYR', 'TYROSINE', 'C9 H11 N O3'),
+                  ('B', 'ASX', 'ASP/ASN AMBIGUOUS', 'C4 H6 N O2 X2'),
+                  ('Z', 'GLX', 'GLU/GLN AMBIGUOUS', 'C5 H8 N O2 X2')])
     _comps['G'] = PeptideChemComp('GLY', 'G', 'G', name='GLYCINE',
                                   formula="C2 H5 N O2")
 
@@ -1177,6 +1202,22 @@ class AsymUnitRange(object):
     entity = property(lambda self: self.asym.entity)
 
 
+class AsymUnitSegment(object):
+    """An aligned part of an asymmetric unit.
+
+       Usually these objects are created from
+       an :class:`AsymUnit`, e.g. to get a segment covering residues 1 through
+       3 in `asym` use::
+
+           asym = ihm.AsymUnit(entity)
+           seg = asym.segment('--ACG', 1, 3)
+    """
+    def __init__(self, asym, gapped_sequence, seq_id_begin, seq_id_end):
+        self.asym = asym
+        self.gapped_sequence = gapped_sequence
+        self.seq_id_range = (seq_id_begin, seq_id_end)
+
+
 class AsymUnit(object):
     """An asymmetric unit, i.e. a unique instance of an Entity that
        was modeled.
@@ -1237,6 +1278,16 @@ class AsymUnit(object):
         """Get a :class:`Residue` at the given sequence position"""
         return Residue(asym=self, seq_id=seq_id)
 
+    def segment(self, gapped_sequence, seq_id_begin, seq_id_end):
+        """Get an object representing the alignment of part of this sequence.
+
+           :param str gapped_sequence: Sequence of the segment, including gaps.
+           :param int seq_id_begin: Start of the segment.
+           :param int seq_id_end: End of the segment.
+        """
+        # todo: cache so we return the same object for same parameters
+        return AsymUnitSegment(self, gapped_sequence, seq_id_begin, seq_id_end)
+
     seq_id_range = property(lambda self: self.entity.seq_id_range,
                             doc="Sequence range")
 
